//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/c/ADLTest.c,v $
// $Revision: 1.6 $
// $Date: 1998/05/12 15:19:16 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment
// NAME:	ADLTest.c
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - Oct 21, 1997
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLTest.h"
#include "ADLResultCodes.h"
#include <stdarg.h>

ADLParameterSet *ADL_params = NULL;

int ADL_skipAssertions = 0;
int ADL_finalJournalReporting = 0;
int ADL_debugMode = 0;

/**
  * Adds a parameter to the test data set for the test. This method 
  * should be called during test case startup by the main routine of
  * an instance of this class - once for each test data parameter.
  */
void ADL_addParameter (ADLDataSet *p) {
  if (ADL_params == NULL) {
      ADL_params = ADL_ParameterSetInit();
  }

  if (ADL_params) {
      ADL__addElement(ADL_params, p);
  }
}

// a static boolean indicating whether the system has been initialized yet or
// not.
static int initialized = 0;

// this is the adlt generated number of instances
int ADL_getNumInstances() {
  char *ptr, *optr;

  if (!initialized) {
      ADL_initialize();
      initialized = 1;
      if (ADL_startup) {
	  ADL_startup();
      }
      ptr = tet_getvar("ADL_RPT_DETAIL");
      if (ptr) {
	  optr = ptr;
	  while (optr[0]) {
	      optr[0] = tolower(optr[0]);
	      optr++;
	  }
	  if (!strcmp(ptr, "long")) {
	      ADL_debugMode = 1;
	  }
      }
  }
  if (ADL_params == NULL) {
    return 0;
  }
  else {
    return ADL__getNumValues(ADL_params);
  }
}


void ADL_runTestInstance(int testIndex) {

  void **parameters;

  // reset skip assertions state
  ADL_skipAssertions = 0;

  // Reset the runtime exception indicator
  ADL_ClearException();
  
  // get parameters from test instance number
  parameters = ADL__provide(ADL_params, testIndex);

  // call the virtual test case procedure
  if (parameters != NULL) {
    ADL_runInstance(parameters);
    ADL_pfree();	/* Free the memory allocated by print functions */
  } else {
    ADL_infoline("Uninitiated test due to provide error");
    ADL_result(ADL_UNINITIATED);
  }

  if (ADL_GetException()) {
      switch (ADL_GetException()) {
      case ADLEndTest:
	// should also notify TET to end the test execution
      default:
	ADL_infoline(ADL_GetExceptionMessage());
      }
  }
  ADL_ClearException();

  if (!ADL__relinquish(ADL_params, testIndex)) {
    ADL_infoline("Relinquish failed!");
  }

  if (ADL_GetException()) {
      switch (ADL_GetException()) {
      case ADLEndTest:
	// should also notify TET to end the test execution
      default:
	ADL_infoline(ADL_GetExceptionMessage());
      }
  }
}


  // Methods for working with the test harness

  /**
   * write an informative entry in the journal.
   *
   * @param line	A String to write.
   */
  void ADL_infoline(char* line) {
    tet_infoline(line);
  }

  /**
   * Get a configuration variable value.
   *
   * @param name	The name of the variable to fetch.
   * @return	A String representing the value of the variable, or NULL if
   * the variable is undefined.
   */

  char* ADL_getvar(char* name) {
    return tet_getvar(name);
  }

  /**
   * Set the block ID for TET. This method should only be called by
   * assertionResult.
   */

  void ADL_setblock() {
    tet_setblock();
  }

  /**
   * Report an intermediate result to TET
   *
   * @param res	The result to record.
   */

  void ADL_result(int res) {
    tet_result(res);
  }

  /**
   * Report the result of an assertion and reset counters to begin recording
   * information about the next assertion.
   *
   * @param res	The result number to use.
   */
  void ADL_assertionStart(char *assertion) {
    ADL_setblock();
    if (ADL_debugMode) {
	ADL_printf("Checking assertion: %s", assertion);
    }
  }

  /**
   * Report the result of an assertion and reset counters to begin recording
   * information about the next assertion.
   *
   * @param res	The result number to use.
   */
  void ADL_assertionResult(int res) {
      if (ADL_debugMode) {
	ADL_printf("result is %d", res);
      }
    ADL_result(res);
  }

  /**
   * Record a result for a factory and an explanation for that result.
   *
   * @param res	The result code.
   * @param	msg	The explanation to record.
   */

  void ADL_factoryResult(int res, char *msg, ...) {
      va_list args;

      va_start(args, msg);
      if (ADL_debugMode) {
	  tet_vprintf(msg, args);
      }
      va_end(msg);;
    ADL_result(res);
  }

  /**
   * Record a result from within a TDD test clause and an explanation for 
   * that result.
   *
   * @param res	The result code.
   * @param	msg	The explanation to record.
   */

  int tdd_result(int res, char* msg, ...) {
      va_list args;

      va_start(args, msg);
      if (ADL_debugMode) {
	  tet_vprintf(msg, args);
      }
      va_end(args);
    ADL_result(res);
    return 1;
  }

  int tdd_assert(char * assertText, int assertRes) {
      if (ADL_debugMode) {
	ADL_printf("Checking tdd assertion: %s", assertRes?"true":"false");
      }
    ADL_result(assertRes ? ADL_PASS : ADL_FAIL);
    return 1;
  }
    
  int tdd_skip() {
    ADL_infoline("Skipping test case due to factory failure");
    ADL_result(ADL_UNINITIATED);

    ADL_skipAssertions = 1;

    return 1;
  }

  int tdd_end_case(char *msg) {
    ADL_skipAssertions = 1;
    if (msg != NULL) {
      ADL_infoline(msg);
    }
    return 1;
  }

  int tdd_end_test() {
    ADL_skipAssertions = 1;
    ADL_infoline("tdd_end_test called");
    return 1;
  }


// ***********************************************************************
// History Log:
// $Log: ADLTest.c,v $
// Revision 1.6  1998/05/12 15:19:16  shane
// Removed extra references to tet_startup and tet_cleanup.
// Added like to makefile to remove ADLTestCase.c after compilation.
//
// Revision 1.5  1998/05/11 12:54:11  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.4  1998/05/08 20:21:08  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.3  1998/05/05 12:54:03  shane
// Added ADL_pfree() call in runTestInstance after the test instance is executed.
//
// Revision 1.2  1998/04/16 12:09:13  shane
// First checkin of complete set.
//
//
// ***********************************************************************
