//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLFactorySet.cpp,v $
// $Revision: 1.9 $
// $Date: 1998/05/28 15:47:51 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   C++ Runtime Environment - Factory set class
// NAME:        ADLFactorySet.cpp
//
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Nov  2, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

#include "ADLFactorySet.hh"
#include "ADLString.hh"
#include <string.h>

ADLFactorySet::ADLFactorySet(ADLTest *tc, char *fname) : 
  ADLDataSet(tc), factoryName(NULL), params(new ADLParameterSet()), 
  factoryParameters(0), providedObject(0) {

  if (fname != NULL) {
    if (factoryName != NULL) delete factoryName;
  
    factoryName = new char[strlen(fname) + 1];
    strcpy(factoryName, fname);
  }
}

void ADLFactorySet::addParameter(ADLDataSet *e) {
  params->addElement(e);
  numValues = params->getNumValues();
}

int ADLFactorySet::getNumParameters() {
  return params->getNumElements();
}

void *ADLFactorySet::provide(int index) throw (ADLRuntimeException) {
  int factoryStatus;

  // check instance number
  checkBounds(index);

  providedObject = 0;

  if (testContext->debugMode)
    testContext->infoline(adlstr.reset() << "Provide for factory " << 
			  factoryName << ":");

  // get the parameter values for the factory method		
  // 
  if (getNumParameters() > 0) {
    factoryParameters = params->provide(index);
  }
  
  try {
     providedObject = factory_provide(factoryParameters);
  }
  catch (ADLRuntimeException&) {
    // propagate an ADLRuntimeException to ADLTest.runTestInstance
    throw;
  }
  catch (...) {
    providedObject = NULL;
    testContext->infoline(adlstr.reset() <<
			  "Caught exception during provide of factory " <<
			  factoryName);
  }

  if (testContext->skipAssertions) {
    // if tdd_skip has been called in provide, status is SKIPPED
    factoryStatus = ADL_SKIPPED;
  } else if (providedObject != NULL) {
    // provide was successful
    factoryStatus = ADL_PASS;
  } else {
    // provide probably failed
    factoryStatus = ADL_FAIL;
    testContext->tdd_skip();
  }
  
  testContext->factoryResult(factoryStatus, 
			     adlstr.reset() << "\tfactory returned: " <<
			     providedObject);
  
  return providedObject;
}

bool ADLFactorySet::relinquish(int index) throw (ADLRuntimeException) {
  bool factoryStatus = true;

  // check instance number
  checkBounds(index);

  if (testContext->debugMode)
    testContext->infoline(adlstr.reset() << "Relinquish for factory " << 
			  factoryName << ":");

  if (providedObject != NULL) {
    try {
      factoryStatus = 
	factory_relinquish(factoryParameters, providedObject);
      if (getNumParameters() > 0) {
	factoryStatus &= params->relinquish(index);
      }
    }
    catch (ADLRuntimeException&) {
      // propagate an ADLRuntimeException to ADLTest.runTestInstance
      throw;
    }
    catch (...) {
      factoryStatus = false;
      testContext->infoline(adlstr.reset() << 
			    "Caught exception during relinquish of factory " << 
			    factoryName);
    }

    providedObject = NULL;
  }
  
  if (factoryParameters != NULL) {
    delete factoryParameters;
    factoryParameters = NULL;
  }

  if (testContext->debugMode)
    testContext->infoline(adlstr.reset() << "\tfactory returned: " <<
			  factoryStatus);

  return factoryStatus;
}

bool ADLFactorySet::factory_relinquish(void** params, void* obj) {
  return true;
}

// ***********************************************************************
// History Log:
// $Log: ADLFactorySet.cpp,v $
// Revision 1.9  1998/05/28 15:47:51  mallet
// Added code to hide certain output unless debugMode is enabled.
//
// Revision 1.8  1998/02/27 10:01:03  mallet
// Updated copyright notice.
//
// Revision 1.7  1998/02/19 09:48:15  mallet
// Changed usage of checkBounds in provide and relinquish according to
// its new signature.
//
// Revision 1.6  1998/02/12 16:04:26  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/02/11 18:12:10  mallet
// Added specific processing for ADLRuntimeException in provide and
// relinquish to allow tdd_end_case and tdd_end_test to act as expected:
// ADLRuntimeException is propagated through.
//
// Revision 1.4  1998/01/29 12:53:45  bareau
// Update of the C++ runtime.
// In ADLFactorySet, factory_relinquish is no more virtual
// but has a default behavior "return true".
// In ADLLiteralSet, added a default constructor with one parameter
// ADLTest *, and a method "void setValues(ADLLiteralSet* copySet)"
// to create an ADLLiteralSet from an other one.
//
// Revision 1.3  1997/11/24 16:07:38  mallet
// Removed inclusion of ADLResultCodes header file (now in ADLTest.hh).
//
// Revision 1.2  1997/11/21 16:49:05  mallet
// Replaced string by char* to become more portable: modified
// initialization of factoryName in constructor.
// Modified reporting for provide and relinquish: the subclasses should
// report on their arguments to have a better report.
// Added exception handling in provide and relinquish.
// Fixed memory leak with factoryParameters array which was never freed.
//
// Revision 1.1  1997/11/14 10:13:29  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
// ***********************************************************************
