//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLTest.cpp,v $
// $Revision: 1.11 $
// $Date: 1998/05/28 15:44:18 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment
// NAME:	ADLTest.cpp
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - Oct 21, 1997
//      See Log at end of file
// 		
// ***********************************************************************

#ifdef GNU_PRAGMA
#ifdef __GNUG__
#pragma implementation "ADLTest.hh"
#endif
#endif

#include "ADLTest.hh"
#include "ADLString.hh"

#ifndef TET_PASS
#include "tet_api.h"
#endif

#ifdef WIN32
#include <string.h>
#define strcasecmp _stricmp
#endif

ADLTest::ADLTest() : skipAssertions(false) {
  params = new ADLParameterSet(); 

  char *debugString = getvar("ADL_RPT_DETAIL");
  debugMode = (debugString != NULL &&
	       (strcasecmp(debugString, "SHORT") != 0));
}

/**
  * Adds a parameter to the test data set for the test. This method 
  * should be called during test case startup by the main routine of
  * an instance of this class - once for each test data parameter.
  */
void ADLTest::addParameter (ADLDataSet *p) {
  params->addElement(p);
}

// this is the adlt generated number of instances
int ADLTest::getNumInstances() {
  if (params == NULL) {
    return 0;
  }
  else {
    return params->getNumValues();
  }
}


void ADLTest::runTestInstance(int testIndex) {
  // reset skip assertions state
  skipAssertions = false;

    try {
      // get parameters from test instance number
      void **parameters = params->provide(testIndex);

      // call the virtual test case procedure
      if (parameters != NULL) {
	runInstance(parameters);
      } else {
	infoline("Uninitiated test due to provide error");
	result(ADL_UNINITIATED);
      }
    }
    catch (ADLRuntimeException e) {
      switch (e.getType()) {
      case ADLEndTest:
	// should also notify TET to end the test execution
      default:
	setblock();
	infoline(e.getMessage());
      }
    }
    catch (...) {
      setblock();
      infoline("Caught exception.");
    }

    setblock();
    try {
      if (!params->relinquish(testIndex)) {
	infoline("Relinquish failed!");
      }
    }
    catch (ADLRuntimeException e) {
      switch (e.getType()) {
      case ADLEndTest:
	// should also notify TET to end the test execution
      default:
	infoline(e.getMessage());
      }
    }
    catch (...) {
      infoline("Caught exception.");
    }
}

// Methods for working with the test harness

/**
  * write an informative entry in the journal.
  *
  * @param line	A String to write.
  */
void ADLTest::infoline(char *line) {
  tet_infoline(line);
}

/**
  * Get a configuration variable value.
  *
  * @param name	The name of the variable to fetch.
  * @return	A String representing the value of the variable, or NULL if
  * the variable is undefined.
  */
char* ADLTest::getvar(char* name) {
  return tet_getvar(name);
}

/**
  * Set the block ID for TET. This method should only be called by
  * assertionResult.
  */
void ADLTest::setblock() {
  tet_setblock();
}

/**
  * Report an intermediate result to TET
  *
  * @param res	The result to record.
  */
void ADLTest::result(int res) {
  tet_result(res);
}

/**
  * Report the result of an assertion and reset counters to begin recording
  * information about the next assertion.
  *
  * @param res	The result number to use.
  */
void ADLTest::assertionStart(char *assertion) {
  if (debugMode) {
    setblock();
    infoline(adlstr.reset() << "Checking assertion: " << assertion);
  }
}

/**
  * Report the result of an assertion and reset counters to begin recording
  * information about the next assertion.
  *
  * @param res	The result number to use.
  */
void ADLTest::assertionResult(int res) {
  if (debugMode)
      infoline(adlstr.reset() << "result is " << res);
  result(res);
 }

/**
  * Record a result for a factory and an explanation for that result.
  *
  * @param res	The result code.
  * @param	msg	The explanation to record.
  */
void ADLTest::factoryResult(int res, char *msg) {
  if (debugMode)
      infoline(msg);
  result(res);
}

/**
  * Record a result from within a TDD test clause and an explanation for 
  * that result.
  *
  * @param res	The result code.
  * @param	msg	The explanation to record.
  */
bool ADLTest::tdd_result(int res, char *msg) {
  if (debugMode)
      infoline(msg);
  result(res);
  return true;
}

bool ADLTest::tdd_assert(char *assertText, bool assertRes) {
  if (debugMode) {
    setblock();
    infoline(adlstr.reset() << "Checking tdd assertion: " << assertText);
  }
  assertionResult(assertRes ? ADL_PASS : ADL_FAIL);
  return assertRes;
}

bool ADLTest::tdd_skip() {
  infoline("Skipping test case due to factory failure");
  result(ADL_SKIPPED);
  
  skipAssertions = true;
  
  return true;
}

bool ADLTest::tdd_end_case(char *msg) throw (ADLRuntimeException) {
  // skip remaining assertions (just in case the thrown exception is
  // catched by the user code)
  skipAssertions = true;

  throw ADLRuntimeException(ADLEndCase, msg);
  return true;
}

bool ADLTest::tdd_end_test() throw (ADLRuntimeException) {
  // skip remaining assertions (just in case the thrown exception is
  // catched by the user code)
  skipAssertions = true;

  throw ADLRuntimeException(ADLEndTest);
  return true;
}

// ***********************************************************************
// History Log:
// $Log: ADLTest.cpp,v $
// Revision 1.11  1998/05/28 15:44:18  mallet
// Added initialization of debugMode indicator through getvar.
// Added some setblock() calls to have correct block numbers during
// relinquish or tdd_assert for instance.
// Added some "if (debugMode)" selectors to reduce the size of the
// journal file when not in "Long" report mode.
//
// Revision 1.10  1998/05/12 17:03:18  shane
// Added handling for debugMode where appropriate to match C and Java runtimes.
// Added debugMode to ADLTest class as a public bool.
//
// Revision 1.9  1998/02/27 10:01:07  mallet
// Updated copyright notice.
//
// Revision 1.8  1998/02/12 16:04:30  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.7  1998/02/11 18:14:05  mallet
// Updated runTestInstance so that exceptions raised during provide and
// relinquish are also caught.
// Changed tdd_skip to set result to ADL_SKIPPED: was ADL_UNINITIATED.
//
// Revision 1.6  1998/01/23 10:06:50  mallet
// Updated tdd_end_case and tdd_end_test to use the new
// ADLRuntimeException class.
// Changed runTestInstance method to implement exception catching
// (thus relieving the generated code from that), in particular in case
// of EndCase and EndTest.
//
// Revision 1.5  1998/01/19 14:37:40  mallet
// Added #ifndef TET_PASS around tet_api.h include to avoid duplicate
// definition of tet types.
// Fixed initialization of tet_testlist in ADLTestCase.cpp for GCC.
//
// Revision 1.4  1997/11/24 16:06:43  mallet
// Moved include of ADLResultCodes in header file to simplify generated
// code which now doesn't need to include ADLResultCodes directly.
// Fixed tdd_assert method to report on the text of the assertion to
// check and to use assertionResult instead of result.
//
// Revision 1.3  1997/11/21 17:04:43  mallet
// Cosmetic changes in reporting methods: 'this.' removed.
//
// Revision 1.2  1997/11/14 10:22:06  mallet
// First compilable version.
// Fixed indentation (cosmetic).
// Fixed return type of tdd_* methods: now use bool.
//
// Revision 1.1  1997/10/22 11:27:58  shane
// Initial version.
//
//
// ***********************************************************************
